@@ -133,7 +133,7 @@ class TextBlock(object):
         min_bbox = np.array([[min_x, min_y, max_x, min_y, max_x, max_y, min_x, max_y]])
         if angled and rotate_back:
             min_bbox = rotate_polygons(center, min_bbox, -self.angle)
-        return min_bbox.reshape(-1, 4, 2)
+        return min_bbox.reshape(-1, 4, 2).astype(np.int64)
 
     # equivalent to qt's boundingRect, ignore angle
     def bounding_rect(self):
@@ -157,13 +157,8 @@ class TextBlock(object):
     def __getitem__(self, idx):
         return self.lines[idx]
 
-    def to_dict(self, extra_info=False):
+    def to_dict(self):
         blk_dict = copy.deepcopy(vars(self))
-        # if not extra_info:
-            # blk_dict.pop('distance')
-            # blk_dict.pop('weight')
-            # blk_dict.pop('vec')
-            # blk_dict.pop('norm')
         return blk_dict
 
     def get_transformed_region(self, img, idx, textheight) -> np.ndarray :
@@ -306,7 +301,7 @@ def sort_textblk_list(blk_list: List[TextBlock], im_w: int, im_h: int) -> List[T
     blk_list.sort(key=lambda blk: blk.weight)
     return blk_list
 
-def examine_textblk(blk: TextBlock, im_w: int, im_h: int, eval_orientation: bool, sort: bool = False) -> None:
+def examine_textblk(blk: TextBlock, im_w: int, im_h: int, sort: bool = False) -> None:
     lines = blk.lines_array()
     middle_pnts = (lines[:, [1, 2, 3, 0]] + lines) / 2
     vec_v = middle_pnts[:, 2] - middle_pnts[:, 0]   # vertical vectors of textlines
@@ -316,7 +311,10 @@ def examine_textblk(blk: TextBlock, im_w: int, im_h: int, eval_orientation: bool
     v = np.sum(vec_v, axis=0)
     h = np.sum(vec_h, axis=0)
     norm_v, norm_h = np.linalg.norm(v), np.linalg.norm(h)
-    vertical = eval_orientation and norm_v > norm_h
+    if blk.language == 'ja':
+        vertical = norm_v > norm_h
+    else:
+        vertical = norm_v > norm_h * 2
     # calcuate distance between textlines and origin 
     if vertical:
         primary_vec, primary_norm = v, norm_v
@@ -336,9 +334,10 @@ def examine_textblk(blk: TextBlock, im_w: int, im_h: int, eval_orientation: bool
     blk.angle = rotation_angle
     if vertical:
         blk.angle -= 90
+    if abs(blk.angle) < 3:
+        blk.angle = 0
     blk.font_size = font_size
-    if eval_orientation:
-        blk.vertical = vertical
+    blk.vertical = vertical
     blk.vec = primary_vec
     blk.norm = primary_norm
     if sort:
@@ -391,21 +390,23 @@ def merge_textlines(blk_list: List[TextBlock]) -> List[TextBlock]:
     return merged_list
 
 def split_textblk(blk: TextBlock):
-    font_size, distance, lines = blk.font_size, blk.distance, blk.lines_array()
+    font_size, distance, lines = blk.font_size, blk.distance, blk.lines
+    l0 = np.array(blk.lines[0])
+    lines.sort(key=lambda line: np.linalg.norm(np.array(line[0]) - l0[0]))
     distance_tol = font_size * 2
     current_blk = copy.deepcopy(blk)
-    current_blk.lines = [lines[0]]
+    current_blk.lines = [l0]
     sub_blk_list = [current_blk]
     textblock_splitted = False
     for jj, line in enumerate(lines[1:]):
         l1, l2 = Polygon(lines[jj]), Polygon(line)
         split = False
         if not l1.intersects(l2):
-            line_disance = distance[jj+1] - distance[jj]
+            line_disance = abs(distance[jj+1] - distance[jj])
             if line_disance > distance_tol:
                 split = True
-            else:
-                if blk.vertical and abs(abs(blk.angle) - 90) < 10:
+            elif blk.vertical and abs(blk.angle) < 15:
+                if len(current_blk.lines) > 1 or line_disance > font_size:
                     split = abs(lines[jj][0][1] - line[0][1]) > font_size
         if split:
             current_blk = copy.deepcopy(current_blk)
@@ -420,8 +421,10 @@ def split_textblk(blk: TextBlock):
     return textblock_splitted, sub_blk_list
 
 def group_output(blks, lines, im_w, im_h, mask=None, sort_blklist=True) -> List[TextBlock]:
-    blk_list, scattered_lines = [], {'ver': [], 'hor': []}
+    blk_list: List[TextBlock] = []
+    scattered_lines = {'ver': [], 'hor': []}
     for bbox, cls, conf in zip(*blks):
+        # cls could give wrong result
         blk_list.append(TextBlock(bbox, language=LANG_LIST[cls]))
 
     # step1: filter & assign lines to textblocks
@@ -445,7 +448,7 @@ def group_output(blks, lines, im_w, im_h, mask=None, sort_blklist=True) -> List[
                 if mask_score < mask_score_thresh:
                     continue
             blk = TextBlock([bx1, by1, bx2, by2], [line])
-            examine_textblk(blk, im_w, im_h, True, sort=False)
+            examine_textblk(blk, im_w, im_h, sort=False)
             if blk.vertical:
                 scattered_lines['ver'].append(blk)
             else:
@@ -453,7 +456,7 @@ def group_output(blks, lines, im_w, im_h, mask=None, sort_blklist=True) -> List[
 
     # step2: filter textblocks, sort & split textlines
     final_blk_list = []
-    for ii, blk in enumerate(blk_list):
+    for blk in blk_list:
         # filter textblocks 
         if len(blk.lines) == 0:
             bx1, by1, bx2, by2 = blk.xyxy
@@ -463,12 +466,15 @@ def group_output(blks, lines, im_w, im_h, mask=None, sort_blklist=True) -> List[
                     continue
             xywh = np.array([[bx1, by1, bx2-bx1, by2-by1]])
             blk.lines = xywh2xyxypoly(xywh).reshape(-1, 4, 2).tolist()
-        lines = blk.lines_array()
-        eval_orientation = blk.language != 'eng'
-        examine_textblk(blk, im_w, im_h, eval_orientation, sort=True)
+        examine_textblk(blk, im_w, im_h, sort=True)
         
         # split manga text if there is a distance gap
-        textblock_splitted = blk.language == 'ja' and len(blk.lines) > 1
+        textblock_splitted = False
+        if len(blk.lines) > 1:
+            if blk.language == 'ja':
+                textblock_splitted = True
+            elif blk.vertical:
+                textblock_splitted = True
         if textblock_splitted:
             textblock_splitted, sub_blk_list = split_textblk(blk)
         else:
@@ -484,6 +490,23 @@ def group_output(blks, lines, im_w, im_h, mask=None, sort_blklist=True) -> List[
     final_blk_list += merge_textlines(scattered_lines['ver'])
     if sort_blklist:
         final_blk_list = sort_textblk_list(final_blk_list, im_w, im_h)
+
+    for blk in final_blk_list:
+        if blk.language == 'eng' and not blk.vertical:
+            num_lines = len(blk.lines)
+            if num_lines == 0:
+                continue
+            # blk.line_spacing = blk.bounding_rect()[3] / num_lines / blk.font_size
+            expand_size = max(int(blk.font_size * 0.1), 2)
+            rad = np.deg2rad(blk.angle)
+            shifted_vec = np.array([[[-1, -1],[1, -1],[1, 1],[-1, 1]]])
+            shifted_vec = shifted_vec * np.array([[[np.sin(rad), np.cos(rad)]]]) * expand_size
+            lines = blk.lines_array() + shifted_vec
+            lines[..., 0] = np.clip(lines[..., 0], 0, im_w-1)
+            lines[..., 1] = np.clip(lines[..., 1], 0, im_h-1)
+            blk.lines = lines.astype(np.int64).tolist()
+            blk.font_size += expand_size
+            
     return final_blk_list
 
 def visualize_textblocks(canvas, blk_list:  List[TextBlock]):
